{% extends 'allianceauth/base.html' %}
{% load i18n %}
{% load static %}
{% block extra_css %}
    <link rel="stylesheet" type="text/css" href="{% static 'buybackprogram/stylse.css' %}">
{% endblock extra_css %}

{% block page_title %}{% translate "buybackprogram" %}{% endblock %}

{% block content %}
    <div class="allianceauth-buyback-program-plugin">
        <div class="col-lg-12 container">
            <h1 class="page-header text-center">{% translate "Buyback Program" %}</h1>
            {% include 'buybackprogram/menu.html' %}
            {% block details %}{% endblock %}
        </div>
    </div>
{% endblock %}
