{% extends 'buybackprogram/page.html' %}
{% load i18n %}
{% load humanize %}
{% load bootstrap %}
{% load static %}

{% block body %}
<div class="panel-heading">
  <h3 class="panel-title">Buyback user settings</h3>
</div>
<div class="panel-body" style="min-height: 100px;">
  <div>
    <form class="form" action="{% url 'buybackprogram:user_settings_edit' %}" method="POST">
      {% csrf_token %}
      {{ form|bootstrap }}
      <button type="submit" class="btn btn-primary">Update Settings</button>
    </form>
  </div>
</div>
{% endblock %}

{% block extra_javascript %}
{% endblock %}

{% block extra_css %}
{% endblock %}

{% block extra_script %}
{% endblock %}
